@@ -6,6 +6,7 @@ from typing import List
 
 import pygit2
 from pathvalidate import sanitize_filepath
+from pygit2 import RemoteCallbacks
 
 from ..Exceptions import ChangesNotCommitedError
 from ..Exceptions import MergeConflict
@@ -106,11 +107,27 @@ class GitLibMixin:
         )
         self.__isIndexDirty = True
 
+    @classmethod
     def cloneGitRepo(
-        self,
+        cls,
+        projectPath: str,
         url: str,
+        username: str = None,
+        password: str = None,
     ):
-        pygit2.clone_repository(url, str(self.projectPath), bare=False)
+        if username is None and password is None:
+            cred = None
+        elif password is None:
+            cred = pygit2.Username(username)
+        else:
+            cred = pygit2.UserPass(username, password)
+
+        pygit2.clone_repository(
+            url,
+            str(projectPath),
+            bare=False,
+            callbacks=RemoteCallbacks(credentials=cred),
+        )
 
     def _writeFile(self, path: str, data: bytes, mode=pygit2.GIT_FILEMODE_BLOB):
         if path != sanitize_filepath(path):
@@ -313,13 +330,19 @@ class GitLibMixin:
     def setCommitterSignature(self, name: str, email: str) -> None:
         self.__committer = pygit2.Signature(name, email)
 
-    def allBranches(self) -> List[str]:
+    def allLocalBranches(self) -> List[str]:
         return [
             str(branch)
-            for branch in self.__repo.listall_branches(pygit2.GIT_BRANCH_ALL)
+            for branch in self.__repo.branches.local
             if not str(branch) == ATTUNE_WORKING_BRANCH
         ]
 
+    def allRemoteBranches(self) -> List[str]:
+        return [str(branch) for branch in self.__repo.branches.remote]
+
+    def allBranches(self) -> List[str]:
+        return [str(branch) for branch in self.__repo.branches]
+
     def allRemotes(self) -> List[tuple[str, str]]:
         return [(remote.name, remote.url) for remote in self.__repo.remotes]
 
@@ -343,7 +366,7 @@ class GitLibMixin:
             # Delete the working branch
             workingBranch.delete()
 
-        if branchName in self.allBranches():
+        if branchName in self.allLocalBranches():
             # Checkout existing branch
             self.__repo.checkout(makeRef(branchName))
         else:
@@ -356,12 +379,13 @@ class GitLibMixin:
     def squashAndMergeWorking(self, mergeCommitMessage: str) -> None:
         working = self.__workingBranch
         checkedOut = self.__checkedOutBranch
+        checkedOutBranchRef = checkedOut.name
 
         if self.isDirty:
             raise ChangesNotCommitedError()
 
         if working.target == checkedOut.target:
-            raise NoChangesToCommitError()
+            raise NoChangesToCommitError("There are no changes to commit")
 
         checkedLastCommit = next(self.__repo.walk(checkedOut.target), None)
 
@@ -401,7 +425,7 @@ class GitLibMixin:
         # Reload the reference to the working branch
         working = self.__workingBranch
 
-        self.__repo.checkout(self.__checkedOutBranchRef)
+        self.__repo.checkout(checkedOutBranchRef)
         checkedOut.set_target(working.target)
         self.__repo.reset(checkedOut.target, pygit2.GIT_RESET_HARD)
 
@@ -418,6 +442,7 @@ class GitLibMixin:
             gitCommit.timestamp = datetime.fromtimestamp(
                 commit.commit_time
             ).strftime("%Y-%m-%d %H:%M:%S")
+            gitCommit.hash = commit.short_id
 
             yield gitCommit
 
@@ -427,6 +452,12 @@ class GitLibMixin:
         else:
             self.__repo.remotes.create(remote, url)
 
+    def deleteRemote(self, remote: str) -> None:
+        if remote in [r[0] for r in self.allRemotes()]:
+            self.__repo.remotes.delete(remote)
+        else:
+            raise ValueError(f"Remote {remote} does not exist")
+
     def pushToRemote(
         self, remote: str, username: str, password: str = None
     ) -> None:
@@ -441,23 +472,28 @@ class GitLibMixin:
         remote = self.__repo.remotes[remote]
 
         remote.push(
-            [makeRef(self.__checkedOutBranch)],
+            [self.__checkedOutBranch.name],
             callbacks=pygit2.RemoteCallbacks(credentials=cred),
         )
 
     def pullFromRemote(
-        self, remote: str, username: str, password: str = None
+        self, remoteName: str, username: str, password: str = None
     ) -> None:
-        if remote not in [r[0] for r in self.allRemotes()]:
-            raise NoRemoteDefined(f"Remote {remote} not set for project")
+        if remoteName not in [r[0] for r in self.allRemotes()]:
+            raise NoRemoteDefined(f"Remote {remoteName} not set for project")
 
         if password is None:
             cred = pygit2.Username(username)
         else:
             cred = pygit2.UserPass(username, password)
 
-        remote = self.__repo.remotes[remote]
-        remote.fetch()
+        remote = self.__repo.remotes[remoteName]
+        remote.fetch(callbacks=RemoteCallbacks(credentials=cred))
+
+        checkedOutBranch = self.__checkedOutBranch
+        remoteBranchName = f"{remoteName}/{checkedOutBranch.shorthand}"
+        # Try to merge remote equivalent into current checked out
+        self.mergeBranches(remoteBranchName)
 
     def mergeBranches(self, sourceBranch: str) -> None:
         if sourceBranch not in self.allBranches():
@@ -471,9 +507,7 @@ class GitLibMixin:
                 f"Squash the commits for {self.__checkedOutBranch} first"
             )
 
-        sourceBranchId = self.__repo.lookup_reference(
-            makeRef(sourceBranch)
-        ).target
+        sourceBranchId = self.__repo.branches[sourceBranch].target
         checkedBranchId = checked.target
 
         mergeAnalysis, _ = self.__repo.merge_analysis(sourceBranchId)
